var phantom = require("selenium-webdriver/phantomjs");
var cheerio = require("cheerio");
var async = require("async");

/*
 * This code finds all relevant prices on an internet page. First all prices are found, then 
 * all strike-through prices are eliminated, as these are original prices.
 */

var driver = new phantom.Driver();
driver.manage().window().setSize(1280, 1024);

var url = "https://www.zalando.nl/anna-field-pantalon-grey-purple-an621a00k-c11.html";

driver.get(url);
findPrices(driver);

function findPrices(driver) {
    
    driver.getPageSource().then(function(html) {
        console.log("finding prices");
        
        var $ = cheerio.load(html);
        var prices = detectPrices($);
        var bprices = getBottomPrices($, prices);
        var realprices = [];
        
        var i=0;
        async.whilst(
            function() {
                return i < bprices.length;
            },
            function(next) {
                isStrikethrough($, bprices[i], driver, function(b) {
                    if (!b) {
                        realprices.push(prices[i]);
                    }
                    i += 1;
                    setImmediate(next);
                });
            },
            function(err) {
                for (i=0; i<realprices.length; i++) {
                    console.log($(realprices[i]).text());
                }
            }
        );
    });
}
    
    
function detectPrices($, tag, isAncestorPrice, isPriceDetected, prices) {

    // Regular expression to detect prices. 
    // see document about detecting prices on the internet
    var priceRegexp = /^(€)?\d+([,.](\d{2}|-))?$/;
    
    // Initialise unassigned variables at first call
    if (typeof(tag) === "undefined") {
        tag = $("body")[0];
    }
    if (typeof(isAncestorPrice) === "undefined") {
        isAncestorPrice = false;
    }
    if (typeof(isPriceDetected) === "undefined") {
        isPriceDetected = false;
    }
    if (typeof(seqPrice) === "undefined") {
        seqPrice = 0;
    }
    if (typeof(prices) === "undefined") {
        prices = [];
    }
    
    // Determine whether "price" is part of the class name 
    // (again, see document about detecting prices on the internet)
    var isTagPrice = tag.attribs && tag.attribs.class && 
        (tag.attribs.class.toLowerCase().indexOf("price") != -1 ||
         tag.attribs.class.toLowerCase().indexOf("prijs") != -1);
    
    // Determine if tag contents constitute a valid price
    var contents = $(tag).text();
    contents = contents.replace(/\s/g, ""); // remove whitespace
    
    var isContentsPrice = priceRegexp.test(contents);
    
    // Determine whether current tag represents a price
    var isPrice = (isAncestorPrice || isTagPrice) && !isPriceDetected && isContentsPrice;
    if (typeof(isPrice) === "undefined") {
        isPrice = false;
    }
    
    if (isPrice) {
        prices.push(tag);
    }
   
    // Do depth-first recursion on the descendants of the current tag
    var children = $(tag).children();
    
    for (var i = 0; i < children.length; i++) {
        prices = detectPrices($, children[i], isAncestorPrice || isTagPrice, isPriceDetected || isPrice, prices);
    }
    
    return prices;
}

function getBottomPrice($, pricetag) {
    
    var price = $(pricetag).text();
    price = price.replace(/\s/g, "");
    
    var children = $(pricetag).children();
    for (var i=0; i<children.length; i++) {
        var newprice = $(children[i]).text();
        newprice = newprice.replace(/\s/g, "")
        
        if (price === newprice) {
            return getBottomPrice($, children[i]);
        }
    }
    
    return pricetag;
}

function getBottomPrices($, prices) {

    var result = [];
    
    for (var i=0; i < prices.length; i++) {
        var bottomPrice = getBottomPrice($, prices[i]);
        result.push(bottomPrice);
    }
    
    return result;
}

function reverseTagSet(tags) {
    
    var result = [];
    
    for (var i = tags.length - 1; i >= 0; i--) {
        result.push(tags[i]);
    }
    
    return result;
}

function generateXPath($, tag) {
    
    var ancestors = reverseTagSet($(tag).parentsUntil());
    ancestors.push(tag);
    var xpath = '/' + ancestors[0].name;
    for (var i=1; i < ancestors.length; i++) {
        var parent = ancestors[i-1];
        var current = ancestors[i];
        var children = $(parent).children(current.name);
        
        for (var j=0; j<children.length; j++) {
            if (children[j] === current) break;
        }
        xpath += '/' + current.name + '[' + (j+1) + ']';
    }
    return xpath;
}

function isStrikethrough($, tag, driver, callback) {
    
    var result = false;
    async.whilst(
        function() {
            return !result && typeof(tag) !== "undefined";
        },
        function(next) {
            var xpath = generateXPath($, tag);
            driver.findElement({xpath: xpath}).getCssValue("text-decoration").then(function(value) {
                if (value.toLowerCase() == "line-through") {
                    result = true;
                }
                tag = $(tag).parent()[0];
                setImmediate(next);
            });
        },
        function(err) {
            callback(result);
        }
    );
}